@@ -1,4 +1,3 @@
-
 ## -----------------------------------------------------------------------
 ## Some operations for Arith and Compare
 ## 6 Compare operation methods, 7 Arith operation methods
@@ -56,10 +55,11 @@ setMethod (
 ## -----------------------------------------------------------------------
 
 ## operation between an db.obj and a single value
+## @param hide.cast for some operations (*, +, -), cast string is not necessary
 .compare <- function (e1, e2, cmp, data.types,
                       prefix = "", res.type = "boolean",
                       cast = "::double precision", res.udt = "bool",
-                      restore.array = TRUE)
+                      restore.array = TRUE, hide.cast = FALSE)
 {
     if (is(e1, "db.data.frame")) e1 <- e1[,]
 
@@ -96,7 +96,8 @@ setMethod (
         if (e1@.col.data_type[i] %in% data.types || is.na(data.types) || whole.array) {
             e2.str <- (if (e2[count %% l + 1] == "") "" else paste(
                 "(", e2[count %% l + 1], ")", sep = ""))
-            expr[i] <- paste(prefix, "(", e1@.expr[i], ")", cast,
+            expr[i] <- paste(prefix, "(", e1@.expr[i], ")",
+                             (if (hide.cast) "" else cast),
                              cmp, e2.str, sep = "")
             count <- count + 1
             i <- i + 1
@@ -122,8 +123,9 @@ setMethod (
                 col.udt_name[i] <- paste("_", res.udt, sep = "")
                 i <- i + 1
             } else {
-                expr.t <- paste(prefix, "(", tmp, ")", cast, cmp, e2.str,
-                                sep = "")
+                expr.t <- paste(prefix, "(", tmp, ")",
+                                (if (hide.cast) "" else cast),
+                                cmp, e2.str, sep = "")
                 expr <- c(expr[seq_len(i-1)], expr.t,
                           expr[(i+1)+seq_len(length(expr)-i)])
                 col.data_type <- c(col.data_type[seq_len(i-1)],
@@ -173,27 +175,11 @@ setMethod (
 
 ## -----------------------------------------------------------------------
 
-.num.types <- c("smallint", "integer", "int2", "int4", "int4",
-                "bigint", "decimal", "numeric", "double precision",
-                "float8", "real", "serial", "bigserial")
-
-## --
-
-.int.types <- c("smallint", "integer", "int2", "int4", "int4",
-                "bigint")
-
-## --
-
-.txt.types <- c("character varying", "varchar", "character",
-                "char", "text")
-
-## -----------------------------------------------------------------------
-
 setMethod (
     ">",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        res <- .compare(e1, e2, " > ", .num.types)
+        res <- .compare(e1, e2, " > ", .num.types, hide.cast = TRUE)
         if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
         res
     },
@@ -215,7 +201,7 @@ setMethod (
     "<",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        res <- .compare(e1, e2, " < ", .num.types)
+        res <- .compare(e1, e2, " < ", .num.types, hide.cast = TRUE)
         if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
         res
     },
@@ -237,7 +223,7 @@ setMethod(
     ">=",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        res <- .compare(e1, e2, " >= ", .num.types)
+        res <- .compare(e1, e2, " >= ", .num.types, hide.cast = TRUE)
         if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
         res
     },
@@ -259,7 +245,7 @@ setMethod (
     "<=",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        res <- .compare(e1, e2, " <= ", .num.types)
+        res <- .compare(e1, e2, " <= ", .num.types, hide.cast = TRUE)
         if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
         res
     },
@@ -281,7 +267,7 @@ setMethod (
     "==",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        res <- .compare(e1, e2, " = ", .num.types)
+        res <- .compare(e1, e2, " = ", .num.types, hide.cast = TRUE)
         if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
         res
     },
@@ -347,7 +333,7 @@ setMethod (
     "!=",
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
-        res <- .compare(e1, e2, " <> ", .num.types)
+        res <- .compare(e1, e2, " <> ", .num.types, hide.cast = TRUE)
         if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
         res
     },
@@ -366,20 +352,34 @@ setMethod (
 ## -----------------------------------------------------------------------
 
 ## convert string to time types
-.replace.timestamp <- function (e1, res, s, op)
+.replace.timestamp <- function (e1, res, s, op, res.type.change = FALSE,
+                                inverse = FALSE, always.interval = FALSE)
 {
-    types <- col.types(e1)
-    time.types <- c("timestamp", "time", "date", "interval")
+    types <- ifelse(e1@.col.data_type == "array",
+                    .strip(e1@.col.udt_name, "_"), e1@.col.udt_name)
+    idx <- match(types, .udt.time.types)
     for (i in seq_len(length(types))) {
-        for (t in time.types) {
-            if (grepl(t, types[i])) {
-                res@.expr[i] <- paste(e1@.expr[i], op, s, "::", t, sep = "")
-                res@.content <- gsub(paste("NULL as \"", res@.col.name[i],
-                                           "\"", sep = ""),
-                                paste(res@.expr[i], " as \"",
-                                      res@.col.name[i], "\"", sep = ""),
-                                res@.content)
-                break
+        if (!is.na(idx[i])) {
+            t <- .time.types[idx[i]]
+            if (always.interval)
+                t1 <- "interval"
+            else
+                t1 <- t
+            if (inverse)
+                res@.expr[i] <- paste(s, "::", t1, op, e1@.expr[i], sep = "")
+            else
+                res@.expr[i] <- paste(e1@.expr[i], op, s, "::", t1, sep = "")
+            res@.content <- gsub(paste("NULL as \"", res@.col.name[i],
+                                       "\"", sep = ""),
+                                 paste(res@.expr[i], " as \"",
+                                       res@.col.name[i], "\"", sep = ""),
+                                 res@.content)
+            if (res.type.change) {
+                res@.col.data_type[i] <- .time.change[idx[i]]
+                res@.col.udt_name[i] <- .time.change[idx[i]]
+            } else {
+                res@.col.data_type[i] <- t
+                res@.col.udt_name[i] <- t
             }
         }
     }
@@ -582,9 +582,24 @@ setMethod (
     signature(e1 = "db.obj", e2 = "numeric"),
     function (e1, e2) {
         res <- .compare(e1, e2, " + ", .num.types,
+                        cast = "",
                         res.type = "double precision",
-                        res.udt = "float8")
+                        res.udt = "float8", hide.cast = TRUE)
         if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+
+        for (i in seq_len(length(names(res)))) {
+            if (grepl("date$", e1@.col.udt_name[i])) {
+                res@.expr[i] <- paste(e1@.expr, " + ", e2, "::integer",
+                                      sep = "")
+                res@.content <- gsub(paste("NULL as \"", res@.col.name[i],
+                                           "\"", sep = ""),
+                                     paste(res@.expr[i], " as \"",
+                                           res@.col.name[i], "\"", sep = ""),
+                                     res@.content)
+                res@.col.data_type[i] <- "date"
+                res@.col.udt_name[i] <- "date"
+            }
+        }
         res
     },
     valueClass = "db.Rquery")
@@ -599,6 +614,32 @@ setMethod (
     },
     valueClass = "db.Rquery")
 
+## --
+setMethod (
+    "+",
+    signature(e1 = "db.obj", e2 = "character"),
+    function (e1, e2) {
+        e2 <- paste("'", .strip(e2, "'"), "'", sep = "")
+        res <- .compare(e1, e2, " + ", .num.types,
+                        cast = "",
+                        res.type = "double precision",
+                        res.udt = "float8")
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res <- .replace.timestamp(e1, res, e2, " + ", always.interval = TRUE)
+        res
+    },
+    valueClass = "db.Rquery")
+
+## --
+
+setMethod (
+    "+",
+    signature(e1 = "character", e2 = "db.obj"),
+    function (e1, e2) {
+        e2 + e1
+    },
+    valueClass = "db.Rquery")
+
 ## --
 
 setMethod (
@@ -607,7 +648,8 @@ setMethod (
     function (e1, e2) {
         res <- .compare(e1, e2, " - ", .num.types,
                         res.type = "double precision",
-                        res.udt = "float8")
+                        res.udt = "float8",
+                        hide.cast = TRUE)
         if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
         res
     },
@@ -615,11 +657,39 @@ setMethod (
 
 ## --
 
+setMethod (
+    "-",
+    signature(e1 = "db.obj", e2 = "character"),
+    function (e1, e2) {
+        e2 <- paste("'", .strip(e2, "'"), "'", sep = "")
+        res <- .compare(e1, e2, " - ", .num.types, cast = "")
+        res <- .replace.timestamp(e1, res, e2, " - ", TRUE)
+        if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
+        res
+    },
+    valueClass = "db.Rquery")
+
+## --
+
+setMethod (
+    "-",
+    signature(e1 = "character", e2 = "db.obj"),
+    function (e1, e2) {
+        e1 <- paste("'", .strip(e1, "'"), "'", sep = "")
+        res <- .compare(e2, e1, " + ", .num.types, prefix = "-", cast = "")
+        res <- .replace.timestamp(e2, res, e1, " - ", TRUE, TRUE)
+        if (is(e2, "db.Rquery")) res@.is.agg <- e2@.is.agg
+        res
+    },
+    valueClass = "db.Rquery")
+
+## --
+
 setMethod (
     "-",
     signature(e1 = "db.obj", e2 = "ANY"),
     function (e1, e2) {
-        if (nargs() == 1) -1 * e1
+        if (nargs() == 1) e1 * (-1)
         else e1 - e2
     },
     valueClass = "db.Rquery")
@@ -631,7 +701,8 @@ setMethod (
     signature(e1 = "numeric", e2 = "db.obj"),
     function (e1, e2) {
         res <- .compare(e2, e1, " + ", .num.types, prefix = "-",
-                        res.type = "double precision", res.udt = "float8")
+                        res.type = "double precision", res.udt = "float8",
+                        hide.cast = TRUE)
         if (is(e2, "db.Rquery")) res@.is.agg <- e2@.is.agg
         res
     },
@@ -678,7 +749,7 @@ setMethod (
         } else {
             res <- .compare(e1, e2, " * ", .num.types,
                             res.type = "double precision",
-                            res.udt = "float8")
+                            res.udt = "float8", hide.cast = TRUE)
             if (is(e1, "db.Rquery")) res@.is.agg <- e1@.is.agg
             res
         }
@@ -716,7 +787,8 @@ setMethod (
     signature(e1 = "numeric", e2 = "db.obj"),
     function (e1, e2) {
         res <- .compare(e2, e1, " * ", .num.types, prefix = "1./",
-                        res.type = "double precision", res.udt = "float8")
+                        res.type = "double precision", res.udt = "float8",
+                        hide.cast = TRUE)
         if (is(e2, "db.Rquery")) res@.is.agg <- e2@.is.agg
         res
     },
@@ -816,7 +888,7 @@ setMethod (
 .operate.two <- function (e1, e2, op, data.types,
                           res.type = "boolean",
                           cast = "::double precision",
-                          res.udt = "bool")
+                          res.udt = "bool", hide.cast = FALSE)
 {
     ## convert db.data.frame into db.Rquery
     if (is(e1, "db.data.frame")) e1 <- e1[,]
@@ -865,26 +937,34 @@ setMethod (
                 tmp2 <- .get.array.elements(e2@.expr[i2], tbl, where.str,
                                         conn.id)
                 if (length(tmp2) != length(tmp1) && length(tmp2) != 1)
-                    stop("Two arrays have to have the same length or one of them has length of 1!")
+                    stop("Two arrays have to have the same length or one ",
+                         "of them has length of 1!")
                 expr[i] <- paste("array[", paste("(", tmp1, ")",
-                                                 cast, op, "(",
+                                                 (if (hide.cast) "" else cast),
+                                                 op, "(",
                                                  tmp2, ")", sep = "",
                                                  collapse = ", "),
                                  "]", sep = "")
             } else {
                 expr[i] <- paste("array[", paste("(", tmp1, ")",
-                                                 cast, op, "(",
+                                                 (if (hide.cast) "" else cast),
+                                                 op, "(",
                                                  e2@.expr[i2], ")",
                                                  sep = "",
                                                  collapse = ", "),
                                  "]", sep = "")
             }
+            col.data_type[i] <- res.type
+            col.udt_name[i] <- res.udt
+            col.name[i] <- paste(names(e1)[i1], "_", names(e2)[i2],
+                                 "_opr", sep = "")
             break
         } else if (e2@.col.data_type[i2] == "array") {
             tmp2 <- .get.array.elements(e2@.expr[i2], tbl, where.str,
                                         conn.id)
             expr[i] <- paste("array[", paste("(", e1@.expr[i1], ")",
-                                             cast, op, "(",
+                                             (if (hide.cast) "" else cast),
+                                             op, "(",
                                              tmp2, ")",
                                              sep = "",
                                              collapse = ", "),
@@ -898,8 +978,8 @@ setMethod (
                 break
             }
         if (v > 0 && e2@.col.data_type[i2] %in% data.types[[v]]) {
-            expr[i] <- paste("(", e1@.expr[i1], ")", cast, op, "(",
-                             e2@.expr[i2], ")", sep = "")
+            expr[i] <- paste("(", e1@.expr[i1], ")", (if (hide.cast) "" else cast),
+                             op, "(", e2@.expr[i2], ")", sep = "")
         } else {
             expr[i] <- "NULL"
         }
@@ -911,6 +991,7 @@ setMethod (
 
     expr.str <- paste(expr, paste("\"", col.name, "\"", sep = ""),
                       sep = " as ", collapse = ", ")
+
     new("db.Rquery",
         .content = paste("select ", expr.str, " from ", tbl,
         where.str, sort$str, sep = ""),
@@ -946,7 +1027,7 @@ setMethod (
         }
         .operate.two(e1, e2, " + ", list(.num.types),
                      res.type = "double precision",
-                     res.udt = "float8")
+                     res.udt = "float8", hide.cast = TRUE)
     },
     valueClass = "db.Rquery")
 
@@ -964,9 +1045,36 @@ setMethod (
             e1 <- unlist(lk(e1))
             return (e1 - e2)
         }
-        .operate.two(e1, e2, " - ", list(.num.types),
-                     res.type = "double precision",
-                     res.udt = "float8")
+
+        .combine.list(sapply(
+            seq_len(length(names(e1))),
+            function(i) {
+                s1 <- sapply(.udt.time.types,
+                             function(s) grepl(paste(s, "$", sep = ""),
+                                               e1@.col.udt_name[i]))
+                s2 <- sapply(.udt.time.types,
+                             function(s) grepl(paste(s, "$", sep = ""),
+                                               e2@.col.udt_name[i]))
+
+                if (any(s1) || any(s2)) {
+                    id <- union(seq_len(length(.time.types))[s1],
+                                seq_len(length(.time.types))[s2])[1]
+                    udt <- (if (.time.change[id] == "integer")
+                            "int4"
+                    else .time.change[id])
+
+                    .operate.two(
+                        e1[[i]], e2[[i]], " - ", list(.time.types),
+                        res.type = .time.change[id],
+                        cast = '',
+                        res.udt = (if (e1@.col.data_type[i] == "array")
+                                   paste("_", udt, sep = "") else udt))
+                } else {
+                    .operate.two(e1[[i]], e2[[i]], " - ", list(.num.types),
+                                 res.type = "double precision",
+                                 res.udt = "float8", hide.cast = TRUE)
+                }
+            }))
     },
     valueClass = "db.Rquery")
 
@@ -1009,9 +1117,15 @@ setMethod (
                    e1@.col.data_type %in% .num.types) {
             return (e2 * e1)
         } else {
-            .operate.two(e1, e2, " * ", list(.num.types),
+            res <- .operate.two(e1, e2, " * ", list(.num.types),
                          res.type = "double precision",
-                         res.udt = "float8")
+                         res.udt = "float8", hide.cast = TRUE)
+            if (length(names(e1)) == 1 && e1@.col.data_type == "array" &&
+                length(names(e2)) == 1 && e2@.col.data_type == "array") {
+                res@.col.data_type <- "array"
+                res@.col.udt_name <- "_float8"
+            }
+            res
         }
     },
     valueClass = "db.Rquery")
